#!/usr/bin/perl

use strict;
use warnings;

die "Stop running as root!!\n" unless $<;

use Getopt::Long;

use Possum::Tags;
my $ptags = new Possum::Tags;

# Defaults {{{
our $separator = "\n";
our $clean = 0;
our $ask = 0;
our $verbose = 0;
our $strip_dir = 0;
our $color = 0;
our $directory = 1;
our @tags = ();
# }}}

do "$ENV{HOME}/.ptags/rc.pl" if -f "$ENV{HOME}/.ptags/rc.pl";

GetOptions(
	"clean" => \$clean,
	"tag|t=s" => \@tags,
	"verbose|v+" => \$verbose,
	"0" => sub { $separator = "\0"},
	"separator=s" => \$separator,
	"strip-dir|s=i" => \$strip_dir,
	"color|c!" => \$color,
	"directory|d!" => \$directory,
	"ask|a" => \$ask);

$ptags->color_enabled(1) if $color;

main->clean if $clean;

# By default, print tags of cwd
main->_cwd_tags && exit unless @ARGV;

if (@tags) {
	my @files = @ARGV;
	foreach my $file (@files) {
		main->tag($file, @tags);
	}
	exit;
}

my $cmd = shift;
main->$cmd(@ARGV) unless $cmd =~ /^_/;

sub clean {
	# Check for nonexistant files
	# Prompt for removal when -a or --ask is passed
	my $self = shift;
	my $files = $ptags->all_files;
	foreach my $file (@$files) {
		unless ( -e $file ) {
			print "$file doesn't exist!\n";
			$self->tags;
			if ($ask) {
				# Is there an easier way to prompt Y/N?
				my $c = 1;
				while ($c) {
					print "Delete entry [Y|n] ? ";
					my $r;
					chomp($r = <STDIN>);
					if ($r) {
						if ($r =~ /^y$/i) {
							print "Deleting...\n";
							$ptags->delete_file($file);
							$c = 0;
						} elsif ( $r =~ /^n$/i ) {
							print "Skipping...\n";
							$c = 0;
						} else {
							print "Please type Y or N...\n";
						}
					} else {
						print "Deleting...\n";
						$ptags->delete_file($file); 
						$c = 0;
					}
				}
			} else {
				print "Deleting...\n";
				$ptags->delete_file($file); 
			}
		}
	}
	exit unless $cmd;
	return 1;
}

sub untag {
	shift;
	my ($tag,@files) = @_;
	$tag = $ptags->colored_tag($tag) if $color;
	if ( $ptags->delete_tag($tag,@files) ) {
		print "$tag deleted from: @files \n" if $verbose;
		return 1;
	}
	print "Error removing $tag from @files \n";
	return;
}

sub tag {
	shift;
	my $file = shift;
	my @tags = @_;
	$ptags->add_tag($file,@tags) or die "Error adding tag";
	$ptags->colored_tags(\@tags) if $color;
	print "Added tags \" @tags \" to file $file successfully!\n";
	1;
}

sub full_tags {
	shift;
	my @file = @_;
	my $tags = $ptags->tags_full_path(@file);
	foreach my $file (keys(%$tags)) {
		$ptags->colored_tags($tags->{$file}) if $color;
		print $file.': '.join(' ',@{$$tags{$file}})."\n";
	}
	1;
}

sub tags {
	shift;
	unless (@_) {
		$ptags->colored_tags($ptags->all_tags) if $color;
		print join($separator,@{$ptags->all_tags}).$separator;
		return 1;
	}
	my @file = @_;
	foreach my $file (@file) {
		my $tags = $ptags->get_tags($file);
		next unless ( @$tags || $verbose);
		$ptags->colored_tags($tags);
		print $file.': '.join(' ',@$tags)."\n";
	}
	1;
}

sub tagged {
	shift;
	my @tags = @_;
	foreach my $tag (@tags) {
		my @jtags = split(/\+/,$tag);
		print "$tag:\n" if $verbose;
		my $files = $ptags->all_with_tag(@jtags);
		unless ($directory) {
			my @files;
			foreach my $file (@$files) {
				push (@files, $file) unless -d  $file;
			}
			$files = \@files;
		}
		@$files = main->_strip_dir(@$files);
		print join($separator,@$files)."$separator";
	}
	1;
}

sub setup {
	my $self = shift;
	mkdir "$ENV{HOME}/.ptags";
	unless ( -f "$ENV{HOME}/.ptags/rc.pl" ) {
		$self->_write_conf("$ENV{HOME}/.ptags/rc.pl");
	} else {
		print "Remove $ENV{HOME}/.ptags/rc.pl to write new default config\n";
	}
	print "Succesful\n" if $ptags->build_table or return;
	1;
} 

sub all {
	shift;
	my $tags = $ptags->full_report;
	foreach my $file ( keys(%$tags) ) {
		my @stripped = main->_strip_dir($file);
		$ptags->colored_tags($tags->{$file}) if $color;
		print shift(@stripped).": ".join(' ',@{$tags->{$file}})."\n";
	}
}

sub help {
	print <<EOF;
PTagS - Possum's Tag System -or- Personal Tag System
Copyright (C) 2007 Possum
[Insert Free License Here]

Usage:
 ptags [-0acdv]
 	Prints tags of current directory.
 ptags [-0acdv] [--separator=n] [--clean] [-t tag] [command [params]]
 	Executes commands (see below)

Commands:
 tag filename tag1 [tag2 [tag3 ...] ]
  	Add tag to file filename
 tagged tag1[+tag1.1] [tag2 [...] ]
  	Print files tagged with specified tags
	Join two or more tags with + to search for files with both
 untag tag [file [...] ]
 	Remove tag from file(s)
 tags [filename [filename2 [...] ]]
  	Print tags associated with filenames
	With no argument, prints all tags
 full_tags tag1 [tag2 [...] ]
  	Like tags but print files with full path
 color 'color string' tags
 	Applies 'color string' to specified tags
	'color string' is like Term::ANSIColor
 setup
 	Creates database files, puts default rc if not exists
EOF
}
	
sub color {
	# Takes color, list of tags, as arguments
	shift;
	my ($color,@tags) = @_;
	foreach my $tag (@tags) {
		$ptags->insert_color($tag,$color) or die "Color invalid?";
	}
}

sub _write_conf {
	shift;
	my $conf = shift;
	open(CONF,">", $conf);
	print CONF <<EOF;
# Configuration file for ptags
#
# Change these values to change ptags' behavior
#
# Defaults {{{
\$separator = "\\n";
\$clean = 0;
\$ask = 0;
\$verbose = 0;
\$strip_dir = 0;
\$color = 0;
\$directory = 1;
# }}}
EOF
	close CONF;
	1;
}

sub _strip_dir {
	# Strip the leading directories to $strip_dir slashes
	shift;
	my @files = @_; # Takes array of filenames
	my @out;
	if ($strip_dir) {
		foreach my $file (@files) {
			$file =~ s{^(\w*?/)\{0,$strip_dir\}}{};
			push (@out,$file);
		}
	} else {
		return @files;
	}
	return @out;
}
			
sub _cwd_tags {
	my $self = shift;
	use Cwd;
	return 1 if $self->tags(Cwd::getcwd);
	return;
}

sub AUTOLOAD {
	print "Unknown or not yet implemented\n";
	print "Please try ptags help\n";
	1;
}
